# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Author: Ole-Johan Skrede                                                    #
# Creation date: 25.10.2016                                                   #
# Revised: 02.11.2016                                                         #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 9, task 2.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import os

import numpy as np
import cv2
import matplotlib.pyplot as plt
import seaborn as sns # pylint: disable=import-error,unused-import

import linear_transforms

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    cv2.imwrite(write_file, image)
  fig_num += 1
  return fig_num

def plot_two_features(class_data, fig_num):
  """Scatter plot of data with features on the axis"""
  labels = ('Class 1', 'Class 2', 'Class 3', 'Class 4')
  colors = ('red', 'blue', 'green', 'yellow')
  markers = ('d', '^', 's', 'o')

  plt.figure(fig_num)
  for c, lab, col, mark in zip(range(4), labels, colors, markers):
    plt.scatter(x=class_data[c][0, :], y=class_data[c][1, :], marker=mark,
                color=col, label=lab, alpha=0.5)
  plt.xlabel('PC 1')
  plt.ylabel('PC 2')

  leg = plt.legend(loc='upper right')
  leg.get_frame().set_alpha(0.5)

  plt.tight_layout()

  fig_num += 1
  return fig_num

def main():
  """main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 9')
  print('Task 1')
  print('-'*80)

  fig_num = 0
  image_dir = '../../images'
  #write_dir = 'results/images'
  feature_filenames = ['tm1.png',
                       'tm2.png',
                       'tm3.png',
                       'tm4.png',
                       'tm5.png',
                       'tm6.png'
                      ]

  feature_images = []
  for fname in feature_filenames:
    feature_images.append(cv2.imread(os.path.join(image_dir, fname), cv2.IMREAD_GRAYSCALE))
  feature_images = np.array(feature_images).astype(np.float32)
  num_features = feature_images.shape[0]

  principal_images, cov_eigvals = linear_transforms.pca(feature_images)

  print(cov_eigvals)
  plt.figure(fig_num)
  plt.plot(cov_eigvals/np.sum(cov_eigvals), 'o')
  plt.ylabel('Normalized eigenvalues')
  fig_num += 1

  plt.figure(fig_num)
  plt.semilogy(cov_eigvals/np.sum(cov_eigvals), 'o')
  plt.ylabel('Normalized eigenvalues (log)')
  fig_num += 1

  for channel in range(feature_images.shape[0]):
    fig_num = plot_image(principal_images[channel, :, :], fig_num)

  # Plot class separation in training mask
  classes = [1, 2, 3, 4]
  training_mask = cv2.imread(os.path.join(image_dir, 'tm_train.png'), cv2.IMREAD_GRAYSCALE)
  pc_class_feature_vectors = []
  for _, class_val in enumerate(classes):
    num_class_pixels = np.count_nonzero(training_mask == class_val)
    class_features = np.zeros([num_features, num_class_pixels])
    for feature_ind in range(num_features):
      pc_image = principal_images[feature_ind, :, :]
      class_features[feature_ind, :] = pc_image[np.nonzero(training_mask == class_val)]
    pc_class_feature_vectors.append(class_features)

  print([vec.shape for vec in pc_class_feature_vectors])
  print([vec.dtype for vec in pc_class_feature_vectors])

  important_features = [vec[:2, :] for vec in pc_class_feature_vectors]
  plot_two_features(important_features, fig_num)

  plt.show()


if __name__ == "__main__":
  main()
